"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AppGraphQl = void 0;
const server_1 = require("@apollo/server");
const server_plugin_landing_page_graphql_playground_1 = require("@apollo/server-plugin-landing-page-graphql-playground");
const express4_1 = require("@apollo/server/express4");
const schema_1 = require("@graphql-tools/schema");
const body_parser_1 = require("body-parser");
const cm_libs_context_aware_error_1 = require("cm-libs-context-aware-error");
const cm_libs_context_aware_logger_1 = require("cm-libs-context-aware-logger");
const cors_1 = __importDefault(require("cors"));
const graphql_1 = require("graphql");
const http_1 = __importDefault(require("http"));
const app_base_1 = require("./app-base");
const app_graphql_directive_1 = require("./app-graphql-directive");
const common_1 = require("./common");
class AppGraphQl extends app_base_1.AppBase {
    _schema;
    async init(params) {
        const { schema, resolvers, ...appBaseInitParams } = params;
        await super.init(appBaseInitParams);
        const transformedSchema = (0, app_graphql_directive_1.applyDirectiveTransformers)((0, schema_1.makeExecutableSchema)({
            typeDefs: [app_graphql_directive_1.directiveTypeDefs, schema],
            resolvers
        }));
        // Keep the full schema
        this._schema = (0, graphql_1.print)(schema);
        // const httpServer = http.createServer(this.app)
        this.httpServer = http_1.default.createServer(this.app);
        const serverConfig = {
            schema: transformedSchema,
            // plugins: [ApolloServerPluginDrainHttpServer({ httpServer: this.httpServer })],
            plugins: [],
            formatError: this.formatError,
            csrfPrevention: true,
            // cache: 'bounded',
            ...appBaseInitParams?.options
        };
        if (this.devMode) {
            serverConfig.plugins.push((0, server_plugin_landing_page_graphql_playground_1.ApolloServerPluginLandingPageGraphQLPlayground)());
        }
        const server = new server_1.ApolloServer(serverConfig);
        await server.start();
        this.app.use('/graphql', (0, cors_1.default)(), (0, body_parser_1.json)(), (0, express4_1.expressMiddleware)(server, {
            context: async ({ req }) => {
                return this.context(req);
            }
        }));
        return this;
    }
    context(req) {
        const ip = (0, common_1.getIp)(req);
        req['logExcluded'] = false;
        if (this.devMode) {
            (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.CRITICAL, () => {
                const headers = JSON.stringify(req.headers, null, 2);
                const body = JSON.stringify(req.body, null, 2);
                return `TRACE REQ: ${req.method} ${req.url}\nHEADERS: ${headers}\nBODY: ${body}`;
            });
        }
        else {
            (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.NORMAL, () => `REQ: ${req.method} ${req.baseUrl} HTTP/${req.httpVersion} @${ip}`);
        }
        return {
            req,
            ip: (0, common_1.getIp)(req),
            accessInfo: (0, common_1.getAuthToken)(req)?.info
        };
    }
    formatError(error) {
        const err = (0, cm_libs_context_aware_error_1.describeError)(error?.originalError || error?.extensions);
        if (error?.path && 0 < error?.path?.length) {
            err['path'] = error.path[0];
        }
        (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.ERROR, () => `GraphQL ERROR @ ${err['path'] || '<PATH>'} - ${err.code}: ${err.reason || err.message || cm_libs_context_aware_error_1.errors.UNKNOWN}`);
        const ret = {
            message: err.message,
            extensions: {
                code: err.code,
                details: error.originalError?.details
            }
        };
        return ret;
    }
    get schema() {
        return this._schema;
    }
}
exports.AppGraphQl = AppGraphQl;
//# sourceMappingURL=app-graphql.js.map